--Copyright (C) 1991-2002 Altera Corporation
--Any megafunction design, and related net list (encrypted or decrypted),
--support information, device programming or simulation file, and any other
--associated documentation or information provided by Altera or a partner
--under Altera's Megafunction Partnership Program may be used only to
--program PLD devices (but not masked PLD devices) from Altera.  Any other
--use of such megafunction design, net list, support information, device
--programming or simulation file, or any other related documentation or
--information is prohibited for any other purpose, including, but not
--limited to modification, reverse engineering, de-compiling, or use with
--any other silicon devices, unless such use is explicitly licensed under
--a separate agreement with Altera or a megafunction partner.  Title to
--the intellectual property, including patents, copyrights, trademarks,
--trade secrets, or maskworks, embodied in any such megafunction design,
--net list, support information, device programming or simulation file, or
--any other related documentation or information provided by Altera or a
--megafunction partner, remains with Altera, the megafunction partner, or
--their respective licensors.  No other licenses, including any licenses
--needed under any third party's intellectual property, are provided herein.
--Copying or modifying any file, or portion thereof, to which this notice
--is attached violates this copyright.

library altera_vhdl_support;
use altera_vhdl_support.altera_vhdl_support_lib.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity timer is 
        port (
              -- inputs:
                 signal address : IN STD_LOGIC_VECTOR (2 DOWNTO 0);
                 signal chipselect : IN STD_LOGIC;
                 signal clk : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
                 signal write_n : IN STD_LOGIC;
                 signal writedata : IN STD_LOGIC_VECTOR (15 DOWNTO 0);

              -- outputs:
                 signal irq : OUT STD_LOGIC;
                 signal readdata : OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
              );

end entity timer;


architecture europa of timer is
              signal clk_en :  STD_LOGIC;
              signal control_continuous :  STD_LOGIC;
              signal control_interrupt_enable :  STD_LOGIC;
              signal control_register :  STD_LOGIC_VECTOR (3 DOWNTO 0);
              signal control_wr_strobe :  STD_LOGIC;
              signal counter_is_running :  STD_LOGIC;
              signal counter_is_zero :  STD_LOGIC;
              signal counter_load_value :  STD_LOGIC_VECTOR (31 DOWNTO 0);
              signal counter_snapshot :  STD_LOGIC_VECTOR (31 DOWNTO 0);
              signal delayed_unxxx40 :  STD_LOGIC;
              signal do_start_counter :  STD_LOGIC;
              signal do_stop_counter :  STD_LOGIC;
              signal force_reload :  STD_LOGIC;
              signal internal_counter :  STD_LOGIC_VECTOR (31 DOWNTO 0);
              signal period_h_register :  STD_LOGIC_VECTOR (15 DOWNTO 0);
              signal period_h_wr_strobe :  STD_LOGIC;
              signal period_l_register :  STD_LOGIC_VECTOR (15 DOWNTO 0);
              signal period_l_wr_strobe :  STD_LOGIC;
              signal read_mux_out :  STD_LOGIC_VECTOR (15 DOWNTO 0);
              signal snap_h_wr_strobe :  STD_LOGIC;
              signal snap_l_wr_strobe :  STD_LOGIC;
              signal snap_read_value :  STD_LOGIC_VECTOR (31 DOWNTO 0);
              signal snap_strobe :  STD_LOGIC;
              signal start_strobe :  STD_LOGIC;
              signal status_wr_strobe :  STD_LOGIC;
              signal stop_strobe :  STD_LOGIC;
              signal timeout_event :  STD_LOGIC;
              signal timeout_occurred :  STD_LOGIC;

begin

  clk_en <= '1';
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      internal_counter <= "00000000000000000100111000100000";
    elsif clk'event and clk = '1' then
      if ((counter_is_running OR force_reload)) = '1' then 
        if ((counter_is_zero OR force_reload)) = '1' then 
          internal_counter <= counter_load_value;
        else
          internal_counter <= internal_counter - "00000000000000000000000000000001";
        end if;
      end if;
    end if;

  end process;

  counter_is_zero <= to_std_logic((internal_counter = "00000000000000000000000000000000"));
  counter_load_value <= period_h_register & period_l_register;
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      force_reload <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        force_reload <= period_h_wr_strobe OR period_l_wr_strobe;
      end if;
    end if;

  end process;

  do_start_counter <= start_strobe;
  do_stop_counter <= stop_strobe OR force_reload OR (counter_is_zero AND NOT control_continuous);
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      counter_is_running <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if do_start_counter = '1' then 
          counter_is_running <= '1';
        elsif do_stop_counter = '1' then 
          counter_is_running <= '0';
        end if;
      end if;
    end if;

  end process;

  --delayed_unxxx40, which is an e_register
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      delayed_unxxx40 <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        delayed_unxxx40 <= counter_is_zero;
      end if;
    end if;

  end process;

  timeout_event <= counter_is_zero AND NOT delayed_unxxx40;
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      timeout_occurred <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if status_wr_strobe = '1' then 
          timeout_occurred <= '0';
        elsif timeout_event = '1' then 
          timeout_occurred <= '1';
        end if;
      end if;
    end if;

  end process;

  irq <= timeout_occurred AND control_interrupt_enable;
  read_mux_out <= ((((((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010")))) AND period_l_register)) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011")))) AND period_h_register))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100"))) & A_ToStdLogicVector(to_std_logic((address = "100")))) AND snap_read_value(15 DOWNTO 0)))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101"))) & A_ToStdLogicVector(to_std_logic((address = "101")))) AND snap_read_value(31 DOWNTO 16)))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001")))) AND ("000000000000" & control_register)))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000")))) AND ("00000000000000" & Std_Logic_Vector'(A_ToStdLogicVector(counter_is_running) & A_ToStdLogicVector(timeout_occurred)))));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      readdata <= "0000000000000000";
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        readdata <= read_mux_out;
      end if;
    end if;

  end process;

  period_l_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "010")))));
  period_h_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "011")))));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      period_l_register <= "0100111000100000";
    elsif clk'event and clk = '1' then
      if period_l_wr_strobe = '1' then 
        period_l_register <= writedata;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      period_h_register <= "0000000000000000";
    elsif clk'event and clk = '1' then
      if period_h_wr_strobe = '1' then 
        period_h_register <= writedata;
      end if;
    end if;

  end process;

  snap_l_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "100")))));
  snap_h_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "101")))));
  snap_strobe <= snap_l_wr_strobe OR snap_h_wr_strobe;
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      counter_snapshot <= "00000000000000000000000000000000";
    elsif clk'event and clk = '1' then
      if snap_strobe = '1' then 
        counter_snapshot <= internal_counter;
      end if;
    end if;

  end process;

  snap_read_value <= counter_snapshot;
  control_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "001")))));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      control_register <= "0000";
    elsif clk'event and clk = '1' then
      if control_wr_strobe = '1' then 
        control_register <= writedata(3 DOWNTO 0);
      end if;
    end if;

  end process;

  stop_strobe <= writedata(3) AND control_wr_strobe;
  start_strobe <= writedata(2) AND control_wr_strobe;
  control_continuous <= control_register(1);
  control_interrupt_enable <= control_register(0);
  status_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "000")))));

end europa;

