--cyclone32: uart.vhd
--manual changes are highlighted by '###'

--Copyright (C) 1991-2002 Altera Corporation
--Any megafunction design, and related net list (encrypted or decrypted),
--support information, device programming or simulation file, and any other
--associated documentation or information provided by Altera or a partner
--under Altera's Megafunction Partnership Program may be used only to
--program PLD devices (but not masked PLD devices) from Altera.  Any other
--use of such megafunction design, net list, support information, device
--programming or simulation file, or any other related documentation or
--information is prohibited for any other purpose, including, but not
--limited to modification, reverse engineering, de-compiling, or use with
--any other silicon devices, unless such use is explicitly licensed under
--a separate agreement with Altera or a megafunction partner.  Title to
--the intellectual property, including patents, copyrights, trademarks,
--trade secrets, or maskworks, embodied in any such megafunction design,
--net list, support information, device programming or simulation file, or
--any other related documentation or information provided by Altera or a
--megafunction partner, remains with Altera, the megafunction partner, or
--their respective licensors.  No other licenses, including any licenses
--needed under any third party's intellectual property, are provided herein.
--Copying or modifying any file, or portion thereof, to which this notice
--is attached violates this copyright.

library altera_vhdl_support;
use altera_vhdl_support.altera_vhdl_support_lib.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity uart_tx is 
        port (
              -- inputs:
                 signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal begintransfer : IN STD_LOGIC;
                 signal clk : IN STD_LOGIC;
                 signal clk_en : IN STD_LOGIC;
                 signal do_force_break : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
                 signal status_wr_strobe : IN STD_LOGIC;
                 signal tx_data : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal tx_wr_strobe : IN STD_LOGIC;

              -- outputs:
                 signal tx_overrun : OUT STD_LOGIC;
                 signal tx_ready : OUT STD_LOGIC;
                 signal tx_shift_empty : OUT STD_LOGIC;
                 signal txd : OUT STD_LOGIC
              );

end entity uart_tx;


architecture europa of uart_tx is
              signal baud_clk_en :  STD_LOGIC;
              signal baud_rate_counter :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal baud_rate_counter_is_zero :  STD_LOGIC;
              signal do_load_shifter :  STD_LOGIC;
              signal do_shift :  STD_LOGIC;
              signal internal_tx_ready :  STD_LOGIC;
              signal pre_txd :  STD_LOGIC;
              signal shift_done :  STD_LOGIC;
              signal tx_load_val :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal tx_shift_reg_out :  STD_LOGIC;
              signal tx_shift_register_contents :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal tx_wr_strobe_onset :  STD_LOGIC;
              signal unxxx48_in :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal unxxx48_out :  STD_LOGIC_VECTOR (9 DOWNTO 0);

begin

  tx_wr_strobe_onset <= tx_wr_strobe AND begintransfer;
  tx_load_val <= Std_Logic_Vector'(A_ToStdLogicVector('1') & tx_data & A_ToStdLogicVector('0'));
  shift_done <= NOT (or_reduce(tx_shift_register_contents));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      do_load_shifter <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        do_load_shifter <= (NOT internal_tx_ready) AND shift_done;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      internal_tx_ready <= '1';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if tx_wr_strobe_onset = '1' then 
          internal_tx_ready <= '0';
        elsif do_load_shifter = '1' then 
          internal_tx_ready <= '1';
        end if;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      tx_overrun <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if status_wr_strobe = '1' then 
          tx_overrun <= '0';
        elsif ((NOT internal_tx_ready AND tx_wr_strobe_onset)) = '1' then 
          tx_overrun <= '1';
        end if;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      tx_shift_empty <= '1';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        tx_shift_empty <= internal_tx_ready AND shift_done;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      baud_rate_counter <= "00000000";
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if ((baud_rate_counter_is_zero OR do_load_shifter)) = '1' then 
          baud_rate_counter <= baud_divisor;
        else
          baud_rate_counter <= baud_rate_counter - "00000001";
        end if;
      end if;
    end if;

  end process;

  baud_rate_counter_is_zero <= to_std_logic((baud_rate_counter = "00000000"));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      baud_clk_en <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        baud_clk_en <= baud_rate_counter_is_zero;
      end if;
    end if;

  end process;

  do_shift <= baud_clk_en AND (NOT shift_done) AND (NOT do_load_shifter);
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      pre_txd <= '1';
    elsif clk'event and clk = '1' then
      if ((NOT shift_done)) = '1' then 
        pre_txd <= tx_shift_reg_out;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      txd <= '1';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        txd <= pre_txd AND NOT do_force_break;
      end if;
    end if;

  end process;

  --_reg, which is an e_register
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      unxxx48_out <= "0000000000";
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        unxxx48_out <= unxxx48_in;
      end if;
    end if;

  end process;

  unxxx48_in <= A_WE_StdLogicVector ((do_load_shifter = '1'),tx_load_val,A_WE_StdLogicVector ((do_shift = '1'),Std_Logic_Vector'(A_ToStdLogicVector('0') & unxxx48_out(9 DOWNTO 1)),unxxx48_out));
  tx_shift_register_contents <= unxxx48_out;
  tx_shift_reg_out <= unxxx48_out(0);
  --vhdl renameroo for output signals
  tx_ready <= internal_tx_ready;

end europa;


library altera_vhdl_support;
use altera_vhdl_support.altera_vhdl_support_lib.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity uart_rx_stimulus_source is 
        port (
              -- inputs:
                 signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal clk : IN STD_LOGIC;
                 signal clk_en : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
                 signal rx_char_ready : IN STD_LOGIC;
                 signal rxd : IN STD_LOGIC;

              -- outputs:
                 signal source_rxd : OUT STD_LOGIC
              );

end entity uart_rx_stimulus_source;


architecture europa of uart_rx_stimulus_source is
component uart_tx is 
           port (
                 -- inputs:
                    signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal begintransfer : IN STD_LOGIC;
                    signal clk : IN STD_LOGIC;
                    signal clk_en : IN STD_LOGIC;
                    signal do_force_break : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal status_wr_strobe : IN STD_LOGIC;
                    signal tx_data : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal tx_wr_strobe : IN STD_LOGIC;

                 -- outputs:
                    signal tx_overrun : OUT STD_LOGIC;
                    signal tx_ready : OUT STD_LOGIC;
                    signal tx_shift_empty : OUT STD_LOGIC;
                    signal txd : OUT STD_LOGIC
                 );
end component uart_tx;

component uart_rx_stimulus_source_character_source_rom_module is 
           port (
                 -- inputs:
                    signal address : IN STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal clk : IN STD_LOGIC;

                 -- outputs:
                    signal q : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
                 );
end component uart_rx_stimulus_source_character_source_rom_module;

              signal aboriginal_start_pulse :  STD_LOGIC;
              signal address :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal address_is_last :  STD_LOGIC;
              signal d1_aboriginal_start_pulse :  STD_LOGIC;
              signal d1_stim_data :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal d2_aboriginal_start_pulse :  STD_LOGIC;
              signal d3_aboriginal_start_pulse :  STD_LOGIC;
              signal d4_aboriginal_start_pulse :  STD_LOGIC;
              signal d5_aboriginal_start_pulse :  STD_LOGIC;
              signal d6_aboriginal_start_pulse :  STD_LOGIC;
              signal d7_aboriginal_start_pulse :  STD_LOGIC;
              signal d8_aboriginal_start_pulse :  STD_LOGIC;
              signal d9_aboriginal_start_pulse :  STD_LOGIC;
              signal delayed_unxxx81 :  STD_LOGIC;
              signal do_send_stim_data :  STD_LOGIC;
              signal internal_source_rxd :  STD_LOGIC;
              signal pickup_pulse :  STD_LOGIC;
              signal startup_pulse :  STD_LOGIC;
              signal stim_data :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal unused_empty :  STD_LOGIC;
              signal unused_overrun :  STD_LOGIC;
              signal unused_ready :  STD_LOGIC;
  constant simulating1 : boolean :=
-- exemplar translate_off
--###{
--    true OR
--###}
-- exemplar translate_on
    false;

begin

--exemplar translate_off
sim_generate:if simulating1 generate
    begin
      stimulus_transmitter : uart_tx
      port map(
        tx_shift_empty => unused_empty,
        txd => internal_source_rxd,
        tx_overrun => unused_overrun,
        tx_ready => unused_ready,
        clk_en => clk_en,
        status_wr_strobe => '0',
        tx_wr_strobe => '1',
        baud_divisor => baud_divisor,
        clk => clk,
        do_force_break => '0',
        tx_data => d1_stim_data,
        begintransfer => do_send_stim_data,
        reset_n => reset_n
      );



      process (clk, reset_n)
    begin
      if reset_n = '0' then
        d1_stim_data <= "00000000";
      elsif clk'event and clk = '1' then
        if do_send_stim_data = '1' then 
          d1_stim_data <= stim_data;
        end if;
      end if;

    end process;


      uart_rx_stimulus_source_character_source_rom : uart_rx_stimulus_source_character_source_rom_module
      port map(
        q => stim_data,
        address => address,
        clk => clk
      );



      process (clk, reset_n)
    begin
      if reset_n = '0' then
        address <= "0000000000";
      elsif clk'event and clk = '1' then
        if do_send_stim_data = '1' then 
          address <= address + "0000000001";
        end if;
      end if;

    end process;


      address_is_last <= to_std_logic((stim_data = "00000000"));

      --delayed_unxxx81, which is an e_register
    process (clk, reset_n)
    begin
      if reset_n = '0' then
        delayed_unxxx81 <= '0';
      elsif clk'event and clk = '1' then
        if clk_en = '1' then 
          delayed_unxxx81 <= rx_char_ready;
        end if;
      end if;

    end process;

    pickup_pulse <= NOT rx_char_ready AND delayed_unxxx81;

      process (clk, reset_n)
    begin
      if reset_n = '0' then
        aboriginal_start_pulse <= '1';
      elsif clk'event and clk = '1' then
        if clk_en = '1' then 
          aboriginal_start_pulse <= '0';
        end if;
      end if;

    end process;


      process (clk, reset_n)
    begin
      if reset_n = '0' then
        d1_aboriginal_start_pulse <= '0';
        d2_aboriginal_start_pulse <= '0';
        d3_aboriginal_start_pulse <= '0';
        d4_aboriginal_start_pulse <= '0';
        d5_aboriginal_start_pulse <= '0';
        d6_aboriginal_start_pulse <= '0';
        d7_aboriginal_start_pulse <= '0';
        d8_aboriginal_start_pulse <= '0';
        d9_aboriginal_start_pulse <= '0';
        startup_pulse <= '0';
      elsif clk'event and clk = '1' then
        if clk_en = '1' then 
          d1_aboriginal_start_pulse <= aboriginal_start_pulse;
          d2_aboriginal_start_pulse <= d1_aboriginal_start_pulse;
          d3_aboriginal_start_pulse <= d2_aboriginal_start_pulse;
          d4_aboriginal_start_pulse <= d3_aboriginal_start_pulse;
          d5_aboriginal_start_pulse <= d4_aboriginal_start_pulse;
          d6_aboriginal_start_pulse <= d5_aboriginal_start_pulse;
          d7_aboriginal_start_pulse <= d6_aboriginal_start_pulse;
          d8_aboriginal_start_pulse <= d7_aboriginal_start_pulse;
          d9_aboriginal_start_pulse <= d8_aboriginal_start_pulse;
          startup_pulse <= d9_aboriginal_start_pulse;
        end if;
      end if;

    end process;


      do_send_stim_data <= (pickup_pulse OR startup_pulse) AND NOT address_is_last;

  end generate sim_generate;
--exemplar translate_on
  synthesis_generate:if NOT simulating1 generate
    begin
      internal_source_rxd <= rxd;

  end generate synthesis_generate;
  --vhdl renameroo for output signals
  source_rxd <= internal_source_rxd;

end europa;


library altera_vhdl_support;
use altera_vhdl_support.altera_vhdl_support_lib.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity uart_rx is 
        port (
              -- inputs:
                 signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal begintransfer : IN STD_LOGIC;
                 signal clk : IN STD_LOGIC;
                 signal clk_en : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
                 signal rx_rd_strobe : IN STD_LOGIC;
                 signal rxd : IN STD_LOGIC;
                 signal status_wr_strobe : IN STD_LOGIC;

              -- outputs:
                 signal break_detect : OUT STD_LOGIC;
                 signal framing_error : OUT STD_LOGIC;
                 signal parity_error : OUT STD_LOGIC;
                 signal rx_char_ready : OUT STD_LOGIC;
                 signal rx_data : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal rx_overrun : OUT STD_LOGIC
              );

end entity uart_rx;


architecture europa of uart_rx is
component uart_rx_stimulus_source is 
           port (
                 -- inputs:
                    signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal clk : IN STD_LOGIC;
                    signal clk_en : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal rx_char_ready : IN STD_LOGIC;
                    signal rxd : IN STD_LOGIC;

                 -- outputs:
                    signal source_rxd : OUT STD_LOGIC
                 );
end component uart_rx_stimulus_source;

              signal baud_clk_en :  STD_LOGIC;
              signal baud_load_value :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal baud_rate_counter :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal baud_rate_counter_is_zero :  STD_LOGIC;
              signal d1_source_rxd :  STD_LOGIC;
              signal delayed_unxxx120 :  STD_LOGIC;
              signal delayed_unxxx89 :  STD_LOGIC;
              signal delayed_unxxx92 :  STD_LOGIC;
              signal do_start_rx :  STD_LOGIC;
              signal got_new_char :  STD_LOGIC;
              signal half_bit_cell_divisor :  STD_LOGIC_VECTOR (6 DOWNTO 0);
              signal internal_rx_char_ready :  STD_LOGIC;
              signal is_break :  STD_LOGIC;
              signal is_framing_error :  STD_LOGIC;
              signal raw_data_in :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal rx_in_process :  STD_LOGIC;
              signal rx_rd_strobe_onset :  STD_LOGIC;
              signal rxd_edge :  STD_LOGIC;
              signal rxd_falling :  STD_LOGIC;
              signal rxd_shift_reg :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal sample_enable :  STD_LOGIC;
              signal shift_reg_start_bit_n :  STD_LOGIC;
              signal source_rxd :  STD_LOGIC;
              signal stop_bit :  STD_LOGIC;
              signal sync_rxd :  STD_LOGIC;
              signal unused_start_bit :  STD_LOGIC;
              signal unxxx108_in :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal unxxx108_out :  STD_LOGIC_VECTOR (9 DOWNTO 0);

begin

  the_uart_rx_stimulus_source : uart_rx_stimulus_source
    port map(
      source_rxd => source_rxd,
      rx_char_ready => internal_rx_char_ready,
      clk_en => clk_en,
      rxd => rxd,
      baud_divisor => baud_divisor,
      clk => clk,
      reset_n => reset_n
    );


  process (clk, reset_n)
  begin
    if reset_n = '0' then
      d1_source_rxd <= '0';
      sync_rxd <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        d1_source_rxd <= source_rxd;
        sync_rxd <= d1_source_rxd;
      end if;
    end if;

  end process;

  --delayed_unxxx89, which is an e_register
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      delayed_unxxx89 <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        delayed_unxxx89 <= sync_rxd;
      end if;
    end if;

  end process;

  rxd_falling <= NOT sync_rxd AND delayed_unxxx89;
  --delayed_unxxx92, which is an e_register
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      delayed_unxxx92 <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        delayed_unxxx92 <= sync_rxd;
      end if;
    end if;

  end process;

  rxd_edge <= sync_rxd XOR delayed_unxxx92;
  rx_rd_strobe_onset <= rx_rd_strobe AND begintransfer;
  half_bit_cell_divisor <= baud_divisor(7 DOWNTO 1);
  baud_load_value <= A_WE_StdLogicVector ((rxd_edge = '1'),("0" & half_bit_cell_divisor),baud_divisor);
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      baud_rate_counter <= "00000000";
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if ((baud_rate_counter_is_zero OR rxd_edge)) = '1' then 
          baud_rate_counter <= baud_load_value;
        else
          baud_rate_counter <= baud_rate_counter - "00000001";
        end if;
      end if;
    end if;

  end process;

  baud_rate_counter_is_zero <= to_std_logic((baud_rate_counter = "00000000"));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      baud_clk_en <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if rxd_edge = '1' then 
          baud_clk_en <= '0';
        else
          baud_clk_en <= baud_rate_counter_is_zero;
        end if;
      end if;
    end if;

  end process;

  sample_enable <= baud_clk_en AND rx_in_process;
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      do_start_rx <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if ((NOT rx_in_process AND rxd_falling)) = '1' then 
          do_start_rx <= '1';
        else
          do_start_rx <= '0';
        end if;
      end if;
    end if;

  end process;

  rx_in_process <= shift_reg_start_bit_n;
  (stop_bit, raw_data_in(7), raw_data_in(6), raw_data_in(5), raw_data_in(4), raw_data_in(3), raw_data_in(2), raw_data_in(1), raw_data_in(0), unused_start_bit) <= rxd_shift_reg;
  is_break <= NOT (or_reduce(rxd_shift_reg));
  is_framing_error <= NOT stop_bit AND NOT is_break;
  --delayed_unxxx120, which is an e_register
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      delayed_unxxx120 <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        delayed_unxxx120 <= rx_in_process;
      end if;
    end if;

  end process;

  got_new_char <= NOT rx_in_process AND delayed_unxxx120;
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      rx_data <= "00000000";
    elsif clk'event and clk = '1' then
      if got_new_char = '1' then 
        rx_data <= raw_data_in;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      framing_error <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if status_wr_strobe = '1' then 
          framing_error <= '0';
        elsif ((got_new_char AND is_framing_error)) = '1' then 
          framing_error <= '1';
        end if;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      break_detect <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if status_wr_strobe = '1' then 
          break_detect <= '0';
        elsif ((got_new_char AND is_break)) = '1' then 
          break_detect <= '1';
        end if;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      rx_overrun <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if status_wr_strobe = '1' then 
          rx_overrun <= '0';
        elsif ((got_new_char AND internal_rx_char_ready)) = '1' then 
          rx_overrun <= '1';
        end if;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      internal_rx_char_ready <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        if rx_rd_strobe_onset = '1' then 
          internal_rx_char_ready <= '0';
        elsif got_new_char = '1' then 
          internal_rx_char_ready <= '1';
        end if;
      end if;
    end if;

  end process;

  parity_error <= '0';
  --_reg, which is an e_register
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      unxxx108_out <= "0000000000";
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        unxxx108_out <= unxxx108_in;
      end if;
    end if;

  end process;

  unxxx108_in <= A_WE_StdLogicVector ((do_start_rx = '1'),Std_Logic_Vector'(A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1') & A_ToStdLogicVector('1')),A_WE_StdLogicVector ((sample_enable = '1'),Std_Logic_Vector'(A_ToStdLogicVector(sync_rxd) & unxxx108_out(9 DOWNTO 1)),unxxx108_out));
  rxd_shift_reg <= unxxx108_out;
  shift_reg_start_bit_n <= unxxx108_out(0);
  --vhdl renameroo for output signals
  rx_char_ready <= internal_rx_char_ready;

end europa;


library altera_vhdl_support;
use altera_vhdl_support.altera_vhdl_support_lib.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

library std;
use std.textio.all;

entity uart_regs is 
        port (
              -- inputs:
                 signal address : IN STD_LOGIC_VECTOR (2 DOWNTO 0);
                 signal break_detect : IN STD_LOGIC;
                 signal chipselect : IN STD_LOGIC;
                 signal clk : IN STD_LOGIC;
                 signal clk_en : IN STD_LOGIC;
                 signal framing_error : IN STD_LOGIC;
                 signal parity_error : IN STD_LOGIC;
                 signal read_n : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
                 signal rx_char_ready : IN STD_LOGIC;
                 signal rx_data : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal rx_overrun : IN STD_LOGIC;
                 signal tx_overrun : IN STD_LOGIC;
                 signal tx_ready : IN STD_LOGIC;
                 signal tx_shift_empty : IN STD_LOGIC;
                 signal write_n : IN STD_LOGIC;
                 signal writedata : IN STD_LOGIC_VECTOR (15 DOWNTO 0);

              -- outputs:
                 signal baud_divisor : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal dataavailable : OUT STD_LOGIC;
                 signal do_force_break : OUT STD_LOGIC;
                 signal irq : OUT STD_LOGIC;
                 signal readdata : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                 signal readyfordata : OUT STD_LOGIC;
                 signal rx_rd_strobe : OUT STD_LOGIC;
                 signal status_wr_strobe : OUT STD_LOGIC;
                 signal tx_data : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal tx_wr_strobe : OUT STD_LOGIC
              );

end entity uart_regs;


architecture europa of uart_regs is
              signal any_error :  STD_LOGIC;
              signal control_reg :  STD_LOGIC_VECTOR (9 DOWNTO 0);
              signal control_wr_strobe :  STD_LOGIC;
              signal cts_status_bit :  STD_LOGIC;
              signal d1_rx_char_ready :  STD_LOGIC;
              signal d1_tx_ready :  STD_LOGIC;
              signal dcts_status_bit :  STD_LOGIC;
              signal delayed_unxxx155 :  STD_LOGIC;
              signal divisor_constant :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal do_write_char :  STD_LOGIC;
              signal eop_status_bit :  STD_LOGIC;
              signal ie_any_error :  STD_LOGIC;
              signal ie_break_detect :  STD_LOGIC;
              signal ie_framing_error :  STD_LOGIC;
              signal ie_parity_error :  STD_LOGIC;
              signal ie_rx_char_ready :  STD_LOGIC;
              signal ie_rx_overrun :  STD_LOGIC;
              signal ie_tx_overrun :  STD_LOGIC;
              signal ie_tx_ready :  STD_LOGIC;
              signal ie_tx_shift_empty :  STD_LOGIC;
              signal internal_tx_data :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal internal_tx_wr_strobe :  STD_LOGIC;
              signal qualified_irq :  STD_LOGIC;
              signal selected_read_data :  STD_LOGIC_VECTOR (15 DOWNTO 0);
              signal status_reg :  STD_LOGIC_VECTOR (12 DOWNTO 0);
  constant simulating2 : boolean :=
-- exemplar translate_off
--###{
--    true OR
--###}
-- exemplar translate_on
    false;

begin

--exemplar translate_off
sim_generate1:if simulating2 generate
    begin
      --delayed_unxxx155, which is an e_register
    process (clk, reset_n)
    begin
      if reset_n = '0' then
        delayed_unxxx155 <= '0';
      elsif clk'event and clk = '1' then
        if clk_en = '1' then 
          delayed_unxxx155 <= tx_ready;
        end if;
      end if;

    end process;

    do_write_char <= tx_ready AND NOT delayed_unxxx155;

      process (clk)
    VARIABLE write_line : line;
    begin
      if clk'event and clk = '1' then
        if do_write_char = '1' then 
          write(write_line, character'val(CONV_INTEGER(internal_tx_data)));
          write(write_line, string'(""));
          write(output, write_line.all);
          deallocate (write_line);
        end if;
      end if;

    end process;


      divisor_constant <= "00000100";

  end generate sim_generate1;
--exemplar translate_on
  synthesis_generate1:if NOT simulating2 generate
    begin
      divisor_constant <= "10101110";

  end generate synthesis_generate1;
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      readdata <= "0000000000000000";
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        readdata <= selected_read_data;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      irq <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        irq <= qualified_irq;
      end if;
    end if;

  end process;

  rx_rd_strobe <= to_std_logic((((((chipselect AND NOT read_n) = '1') AND (address = "000")))));
  internal_tx_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "001")))));
  status_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "010")))));
  control_wr_strobe <= to_std_logic((((((chipselect AND NOT write_n) = '1') AND (address = "011")))));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      internal_tx_data <= "00000000";
    elsif clk'event and clk = '1' then
      if internal_tx_wr_strobe = '1' then 
        internal_tx_data <= writedata(7 DOWNTO 0);
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      control_reg <= "0000000000";
    elsif clk'event and clk = '1' then
      if control_wr_strobe = '1' then 
        control_reg <= writedata(9 DOWNTO 0);
      end if;
    end if;

  end process;

  baud_divisor <= divisor_constant;
  cts_status_bit <= '0';
  dcts_status_bit <= '0';
  (do_force_break, ie_any_error, ie_rx_char_ready, ie_tx_ready, ie_tx_shift_empty, ie_tx_overrun, ie_rx_overrun, ie_break_detect, ie_framing_error, ie_parity_error) <= control_reg;
  any_error <= tx_overrun OR rx_overrun OR parity_error OR framing_error OR break_detect;
  status_reg <= Std_Logic_Vector'(A_ToStdLogicVector(eop_status_bit) & A_ToStdLogicVector(cts_status_bit) & A_ToStdLogicVector(dcts_status_bit) & A_ToStdLogicVector('0') & A_ToStdLogicVector(any_error) & A_ToStdLogicVector(rx_char_ready) & A_ToStdLogicVector(tx_ready) & A_ToStdLogicVector(tx_shift_empty) & A_ToStdLogicVector(tx_overrun) & A_ToStdLogicVector(rx_overrun) & A_ToStdLogicVector(break_detect) & A_ToStdLogicVector(framing_error) & A_ToStdLogicVector(parity_error));
  process (clk, reset_n)
  begin
    if reset_n = '0' then
      d1_rx_char_ready <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        d1_rx_char_ready <= rx_char_ready;
      end if;
    end if;

  end process;

  process (clk, reset_n)
  begin
    if reset_n = '0' then
      d1_tx_ready <= '0';
    elsif clk'event and clk = '1' then
      if clk_en = '1' then 
        d1_tx_ready <= tx_ready;
      end if;
    end if;

  end process;

  dataavailable <= d1_rx_char_ready;
  readyfordata <= d1_tx_ready;
  eop_status_bit <= '0';
  selected_read_data <= ((((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000"))) & A_ToStdLogicVector(to_std_logic((address = "000")))) AND ("00000000" & rx_data))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001"))) & A_ToStdLogicVector(to_std_logic((address = "001")))) AND ("00000000" & internal_tx_data)))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010"))) & A_ToStdLogicVector(to_std_logic((address = "010")))) AND ("000" & status_reg)))) OR ((Std_Logic_Vector'(A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011"))) & A_ToStdLogicVector(to_std_logic((address = "011")))) AND ("000000" & control_reg)));
  qualified_irq <= (ie_any_error AND any_error) OR (ie_tx_shift_empty AND tx_shift_empty) OR (ie_tx_overrun AND tx_overrun) OR (ie_rx_overrun AND rx_overrun) OR (ie_break_detect AND break_detect) OR (ie_framing_error AND framing_error) OR (ie_parity_error AND parity_error) OR (ie_rx_char_ready AND rx_char_ready) OR (ie_tx_ready AND tx_ready);
  --vhdl renameroo for output signals
  tx_wr_strobe <= internal_tx_wr_strobe;
  --vhdl renameroo for output signals
  tx_data <= internal_tx_data;

end europa;


library altera_vhdl_support;
use altera_vhdl_support.altera_vhdl_support_lib.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity uart is 
        port (
              -- inputs:
                 signal address : IN STD_LOGIC_VECTOR (2 DOWNTO 0);
                 signal begintransfer : IN STD_LOGIC;
                 signal chipselect : IN STD_LOGIC;
                 signal clk : IN STD_LOGIC;
                 signal read_n : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
                 signal rxd : IN STD_LOGIC;
                 signal write_n : IN STD_LOGIC;
                 signal writedata : IN STD_LOGIC_VECTOR (15 DOWNTO 0);

              -- outputs:
                 signal dataavailable : OUT STD_LOGIC;
                 signal irq : OUT STD_LOGIC;
                 signal readdata : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                 signal readyfordata : OUT STD_LOGIC;
                 signal txd : OUT STD_LOGIC
              );

end entity uart;


architecture europa of uart is
component uart_tx is 
           port (
                 -- inputs:
                    signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal begintransfer : IN STD_LOGIC;
                    signal clk : IN STD_LOGIC;
                    signal clk_en : IN STD_LOGIC;
                    signal do_force_break : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal status_wr_strobe : IN STD_LOGIC;
                    signal tx_data : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal tx_wr_strobe : IN STD_LOGIC;

                 -- outputs:
                    signal tx_overrun : OUT STD_LOGIC;
                    signal tx_ready : OUT STD_LOGIC;
                    signal tx_shift_empty : OUT STD_LOGIC;
                    signal txd : OUT STD_LOGIC
                 );
end component uart_tx;

component uart_rx is 
           port (
                 -- inputs:
                    signal baud_divisor : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal begintransfer : IN STD_LOGIC;
                    signal clk : IN STD_LOGIC;
                    signal clk_en : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal rx_rd_strobe : IN STD_LOGIC;
                    signal rxd : IN STD_LOGIC;
                    signal status_wr_strobe : IN STD_LOGIC;

                 -- outputs:
                    signal break_detect : OUT STD_LOGIC;
                    signal framing_error : OUT STD_LOGIC;
                    signal parity_error : OUT STD_LOGIC;
                    signal rx_char_ready : OUT STD_LOGIC;
                    signal rx_data : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal rx_overrun : OUT STD_LOGIC
                 );
end component uart_rx;

component uart_regs is 
           port (
                 -- inputs:
                    signal address : IN STD_LOGIC_VECTOR (2 DOWNTO 0);
                    signal break_detect : IN STD_LOGIC;
                    signal chipselect : IN STD_LOGIC;
                    signal clk : IN STD_LOGIC;
                    signal clk_en : IN STD_LOGIC;
                    signal framing_error : IN STD_LOGIC;
                    signal parity_error : IN STD_LOGIC;
                    signal read_n : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal rx_char_ready : IN STD_LOGIC;
                    signal rx_data : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal rx_overrun : IN STD_LOGIC;
                    signal tx_overrun : IN STD_LOGIC;
                    signal tx_ready : IN STD_LOGIC;
                    signal tx_shift_empty : IN STD_LOGIC;
                    signal write_n : IN STD_LOGIC;
                    signal writedata : IN STD_LOGIC_VECTOR (15 DOWNTO 0);

                 -- outputs:
                    signal baud_divisor : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal dataavailable : OUT STD_LOGIC;
                    signal do_force_break : OUT STD_LOGIC;
                    signal irq : OUT STD_LOGIC;
                    signal readdata : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                    signal readyfordata : OUT STD_LOGIC;
                    signal rx_rd_strobe : OUT STD_LOGIC;
                    signal status_wr_strobe : OUT STD_LOGIC;
                    signal tx_data : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal tx_wr_strobe : OUT STD_LOGIC
                 );
end component uart_regs;

              signal baud_divisor :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal break_detect :  STD_LOGIC;
              signal clk_en :  STD_LOGIC;
              signal do_force_break :  STD_LOGIC;
              signal framing_error :  STD_LOGIC;
              signal internal_dataavailable :  STD_LOGIC;
              signal internal_irq :  STD_LOGIC;
              signal internal_readdata :  STD_LOGIC_VECTOR (15 DOWNTO 0);
              signal internal_readyfordata :  STD_LOGIC;
              signal internal_txd :  STD_LOGIC;
              signal parity_error :  STD_LOGIC;
              signal rx_char_ready :  STD_LOGIC;
              signal rx_data :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal rx_overrun :  STD_LOGIC;
              signal rx_rd_strobe :  STD_LOGIC;
              signal status_wr_strobe :  STD_LOGIC;
              signal tx_data :  STD_LOGIC_VECTOR (7 DOWNTO 0);
              signal tx_overrun :  STD_LOGIC;
              signal tx_ready :  STD_LOGIC;
              signal tx_shift_empty :  STD_LOGIC;
              signal tx_wr_strobe :  STD_LOGIC;

begin

  clk_en <= '1';
  the_uart_tx : uart_tx
    port map(
      tx_shift_empty => tx_shift_empty,
      txd => internal_txd,
      tx_overrun => tx_overrun,
      tx_ready => tx_ready,
      clk_en => clk_en,
      status_wr_strobe => status_wr_strobe,
      tx_wr_strobe => tx_wr_strobe,
      baud_divisor => baud_divisor,
      clk => clk,
      do_force_break => do_force_break,
      tx_data => tx_data,
      begintransfer => begintransfer,
      reset_n => reset_n
    );


  the_uart_rx : uart_rx
    port map(
      framing_error => framing_error,
      parity_error => parity_error,
      rx_char_ready => rx_char_ready,
      break_detect => break_detect,
      rx_overrun => rx_overrun,
      rx_data => rx_data,
      clk_en => clk_en,
      status_wr_strobe => status_wr_strobe,
      rxd => rxd,
      baud_divisor => baud_divisor,
      clk => clk,
      begintransfer => begintransfer,
      rx_rd_strobe => rx_rd_strobe,
      reset_n => reset_n
    );


  the_uart_regs : uart_regs
    port map(
      readdata => internal_readdata,
      status_wr_strobe => status_wr_strobe,
      tx_wr_strobe => tx_wr_strobe,
      baud_divisor => baud_divisor,
      readyfordata => internal_readyfordata,
      dataavailable => internal_dataavailable,
      do_force_break => do_force_break,
      irq => internal_irq,
      tx_data => tx_data,
      rx_rd_strobe => rx_rd_strobe,
      framing_error => framing_error,
      rx_char_ready => rx_char_ready,
      clk_en => clk_en,
      rx_overrun => rx_overrun,
      chipselect => chipselect,
      read_n => read_n,
      clk => clk,
      tx_overrun => tx_overrun,
      tx_shift_empty => tx_shift_empty,
      writedata => writedata,
      parity_error => parity_error,
      break_detect => break_detect,
      address => address,
      rx_data => rx_data,
      write_n => write_n,
      reset_n => reset_n,
      tx_ready => tx_ready
    );


  --vhdl renameroo for output signals
  readdata <= internal_readdata;
  --vhdl renameroo for output signals
  readyfordata <= internal_readyfordata;
  --vhdl renameroo for output signals
  dataavailable <= internal_dataavailable;
  --vhdl renameroo for output signals
  txd <= internal_txd;
  --vhdl renameroo for output signals
  irq <= internal_irq;

end europa;

